<?php

namespace SweepAndGo\App\Helpers;

class Location extends SngHelper {
	protected string $table = '';
	protected string $location_settings = '';

	public function __construct() {
		parent::__construct();

		$this->table = $this->db->prefix . SNG_TABLE_PREFIX . 'locations';
		$this->location_settings = $this->db->prefix . SNG_TABLE_PREFIX . 'location_settings';
	}

	public function all(): ?array {
		$query = "SELECT * FROM $this->table";

		return $this->db->get_results($query, OBJECT);
	}

	public function all_with_settings( array $settings ): ?array {

		$query = "SELECT * FROM $this->table";

		$data = $this->db->get_results($query, OBJECT);

		array_map(function ($value) use ($settings) {
			$value->settings = (new LocationSetting())->get_settings_for_location($value->id, $settings);
			return $value;
		}, $data);

		return $data;
	}

	public function get( int $id ) {
		return $this->db->get_row($this->db->prepare("SELECT * FROM $this->table WHERE id = %d LIMIT 1", $id), OBJECT);
	}

	public function first( string $field, string $operator, string $value) {
		$query = $this->db->prepare( "SELECT * FROM $this->table WHERE $field $operator %s LIMIT 1", $value );
		return $this->db->get_row($query, OBJECT);
	}

	public function create($data) {
		$create = $this->db->insert($this->table, [
			...$data,
			'updated_at' => current_time('mysql'),
			'created_at' => current_time('mysql')
		]);

		if ( $create ) {
			return $this->db->insert_id;
		} else {
			return new \WP_Error(500, $this->db->last_error);
		}

	}

	public function delete( $id ) {
		$location = $this->db->delete( $this->table, ['id' => $id] );

		return $location;
	}
}