<?php

namespace SweepAndGo\App\Helpers;

class LocationSetting extends SngHelper {
	protected string $table = '';

	public function __construct() {
		parent::__construct();

		$this->table = $this->db->prefix . SNG_TABLE_PREFIX . 'location_settings';
	}

	public function all(): ?array {
		$query = "SELECT * FROM $this->table";

		return $this->db->get_results($query, OBJECT);
	}

	public function create($data) {
		$this->db->insert($this->table, $data );

		return $this->db->insert_id;
	}

	public function delete( $location_id ) {
		$location_setting = $this->db->delete( $this->table, ['location_id' => $location_id] );

		return $location_setting;
	}

	public function get_settings_for_location( int $location_id, array $setting_keys = [] ): array {
		if ( empty( $setting_keys ) ) {
			$setting_keys = [
				'production',
				'onboarding_form_enabled',
				'pages',
				'onboarding_form',
				'payment_processor',
				'service_cross_sells',
				'openapi_token',
			];
		}

		$settings_filter_placeholder = implode( ', ', array_fill( 0, count( $setting_keys ), '%s' ) );
		$setting_keys['location_id'] = $location_id;
		$data = $this->db->get_results(
			$this->db->prepare( "SELECT setting_key, setting_value FROM $this->table WHERE setting_key IN ($settings_filter_placeholder)
				AND location_id = %d", $setting_keys ) );
		array_map(function ($value) {
			$value->setting_value = maybe_unserialize($value->setting_value);
			return $value;
		}, $data);

		$data = array_combine(wp_list_pluck($data, 'setting_key'), wp_list_pluck($data, 'setting_value'));

		return $data;
	}

	/**
	 * @return array
	 */
	public function get_location_settings_page_data(): array {
		$data = $this->db->get_results("SELECT setting_value FROM $this->table WHERE setting_key = 'onboarding_form'", OBJECT);

		return $data;
	}

	/**
	 */
	public function get_location_setting_single(int $location_id, string $key) {
		$query = $this->db->prepare("SELECT setting_value FROM {$this->table} WHERE location_id = %d 
			AND setting_key = %s", $location_id, $key);

		return maybe_unserialize( $this->db->get_var($query) );
	}

	public function get_onboarding_form_overwrites_option() {
		return get_option('onboarding_form_overwrites');
	}

	public function get_service_area_checker_overwrites_option() {
		return get_option('sng_sac_options');
	}

	public function get_email_notifications_option() {
		return get_option('sng_email_settings');
	}

	public function update( $key, $data, $id ) {
		$update = $this->db->update( $this->table,
			[
				'setting_value' => maybe_serialize( $data ),
				'updated_at' => current_time('mysql')
			],
			[
				'setting_key' => $key,
				'location_id' => $id
			]
		);

		if ( ! $update ) {
			return [
				'error' => $this->db->last_error,
				'query' => $this->db->last_query
			];
		}

		return $update;
	}

	public function getTokens() {
		$data = $this->db->get_results("SELECT setting_value as token, location_id FROM 
                                               $this->table WHERE setting_key = 'openapi_token'", OBJECT);

		return $data;
	}
}
