<?php

namespace SweepAndGo\App\Hooks;

use Cassandra\Set;
use SweepAndGo\App\Http\Controllers\AddonController;
use SweepAndGo\App\Http\Controllers\LocationsController;
use SweepAndGo\App\Http\Controllers\OauthHandlerController;
use SweepAndGo\App\Http\Controllers\SettingsController;

class MenuHook {
	private $page_prefix = 'sweepandgo';

	private $pages = [
		'sweepandgo-locations',
		'sweepandgo-addons',
		'sweepandgo-settings',
		'sweepandgo-logs',

	];

	private LocationsController $locations_controller;
	private AddonController $addon_controller;
	private OauthHandlerController $oauth_handler_controller;
	private SettingsController $settings_controller;

	public function __construct(
		LocationsController $locations_controller,
		AddonController $addon_controller,
		OauthHandlerController $oauth_handler_controller,
		SettingsController $settings_controller
	) {
		$this->locations_controller     = $locations_controller;
		$this->addon_controller         = $addon_controller;
		$this->oauth_handler_controller = $oauth_handler_controller;
		$this->settings_controller      = $settings_controller;
	}

	public function handle() {
		add_action( 'admin_menu', [ $this, 'add_menu' ] );
		add_action( 'sng_settings_menu_pages', [ $this, 'sng_settings_menu_pages' ], 1, 2 );
		add_action( 'admin_enqueue_scripts', [ $this, 'enqueue_assets' ] );
	}

	public function add_menu() {
		/**
		 * Parent menu
		 */
		add_menu_page(
			'Sweep&Go',
			'Sweep&Go',
			'manage_options',
			"{$this->page_prefix}-locations",
			[
				$this->locations_controller,
				'index'
			],
			SNG_PLUGIN_URL_PATH . 'assets/images/logo.png',
			2
		);

		/**
		 * View/add locations
		 */
		add_submenu_page(
			"{$this->page_prefix}-locations",
			'Locations',
			'Locations',
			'manage_options',
			"{$this->page_prefix}-locations",
			[
				$this->locations_controller,
				'index'
			],
		);


		/**
		 * sng_page Posts
		 */
		add_submenu_page(
			"{$this->page_prefix}-locations",
			'Pages',
			'Pages',
			'manage_options',
			'edit.php?post_type=sng_page'
		);

		/**
		 * Addons
		 */
		add_submenu_page(
			"{$this->page_prefix}-locations",
			'Add-Ons',
			'Add-Ons',
			'manage_options',
			"{$this->page_prefix}-addons",
			[
				$this->addon_controller,
				'addons'
			],
		);

		/**
		 * Logs
		 */
		add_submenu_page(
			"{$this->page_prefix}-locations",
			'Logs',
			'Logs',
			'manage_options',
			"{$this->page_prefix}-logs",
			[
				$this->locations_controller,
				'logs'
			],
		);

		/**
		 * Plugin Settings
		 */
		add_submenu_page(
			"{$this->page_prefix}-locations",
			'Settings',
			'Settings',
			'manage_options',
			"{$this->page_prefix}-settings",
			[
				$this->settings_controller,
				'settings'
			],
		);

		/**
		 * Page for oauth connection
		 */
		add_submenu_page(
			'hidden',
			'Sweep&Go Oauth Connection',
			'Sweep&Go Oauth Connection',
			'manage_options',
			"{$this->page_prefix}-oauth-connect",
			[
				$this->oauth_handler_controller,
				'oauth_connect'
			],
		);

		/**
		 * AB Testing Handler
		 */
		add_submenu_page(
			'hidden',
			'Sweep&Go Client Onboarding AB Testing',
			'Sweep&Go Client Onboarding AB Testing',
			'manage_options',
			"{$this->page_prefix}-ab-testing",
			[
				$this->settings_controller,
				'ab_testing'
			],
		);
		add_submenu_page(
			'hidden',
			'Sweep&Go Client Onboarding AB Testing Finalize',
			'Sweep&Go Client Onboarding AB Testing Finalize',
			'manage_options',
			"{$this->page_prefix}-ab-testing-finalize",
			[
				$this->settings_controller,
				'ab_testing_finalize'
			],
		);
	}

	public function enqueue_assets() {
		wp_enqueue_style( 'sweep-and-go-style', SNG_PLUGIN_URL_PATH . 'assets/admin/css/sweep-and-go.css', [], SNG_VERSION, 'all' );
		wp_enqueue_style( 'sng-select2-css', SNG_PLUGIN_URL_PATH . 'assets/libs/css/select2.min.css', [], SNG_VERSION, 'all' );
		wp_enqueue_style( 'sng-icons', SNG_PLUGIN_URL_PATH . 'assets/css/sng-icons.css', [], SNG_VERSION, 'all' );
		do_action('sng_spinner_element');
		wp_enqueue_style( 'sng-spinner-css', SNG_PLUGIN_URL_PATH . 'assets/css/spinner.css', [], SNG_VERSION, 'all' );

		/**
		 * Scripts
		 */
		wp_enqueue_script( 'sng-select2-js', SNG_PLUGIN_URL_PATH . 'assets/libs/js/select2.min.js', [ 'jquery' ], SNG_VERSION, true );
		wp_enqueue_script( 'sweep-an-go-admin-main', SNG_PLUGIN_URL_PATH . 'assets/admin/js/main.js', [ 'jquery' ], SNG_VERSION, true );
		wp_localize_script( 'sweep-an-go-admin-main', "ajax_wp", [
			"ajax_url" => admin_url( 'admin-ajax.php' ),
			"nonce"    => wp_create_nonce( 'admin-ajax-nonce' ),
		] );
		wp_scripts()->add_data( 'sweep-an-go-admin-main', 'type', 'module' );
	}
}