<?php

namespace SweepAndGo\App\Hooks;

use SweepAndGo\App\Helpers\Log;
use SweepAndGo\App\Http\Controllers\AddonController;
use SweepAndGo\App\Http\Controllers\LocationsController;
use SweepAndGo\App\Http\Controllers\LogsController;
use SweepAndGo\App\Http\Controllers\SettingsController;

class RequestsHook {
	private string $prefix = "sng";

	private LocationsController $locations_controller;

	private LogsController $logs_controller;
	private AddonController $addon_controller;
	private SettingsController $settings_controller;

	private array $routes = [];

	protected array $filters = [];

	public function __construct(
		LocationsController $locations_controller,
		LogsController $logs_controller,
		AddonController $addon_controller,
		SettingsController $settings_controller,
	) {
		$this->locations_controller = $locations_controller;
		$this->logs_controller      = $logs_controller;
		$this->addon_controller     = $addon_controller;
		$this->settings_controller  = $settings_controller;
		$this->routes               = [
			[
				'name'     => "{$this->prefix}_install_addon",
				'callback' => [ $this->addon_controller, 'install_addon' ],
				'type'     => 'wp_ajax'
			],
			[
				'name'     => "{$this->prefix}_deactivate_addon",
				'callback' => [ $this->addon_controller, 'deactivate_addon' ],
				'type'     => 'wp_ajax'
			],
			[
				'name'     => "{$this->prefix}_activate_addon",
				'callback' => [ $this->addon_controller, 'activate_addon' ],
				'type'     => 'wp_ajax'
			],
			[
				'name'     => "{$this->prefix}_connect_location",
				'callback' => [ $this->locations_controller, 'connect' ],
				'type'     => 'admin_post'
			],
			[
				'name'     => "{$this->prefix}_delete_location",
				'callback' => [ $this->locations_controller, 'delete' ],
				'type'     => 'admin_post'
			],
			[
				'name'     => "{$this->prefix}_settings_update",
				'callback' => [ $this->settings_controller, 'settings_update' ],
				'type'     => 'admin_post'
			],
			[
				'name'     => "{$this->prefix}_settings_reset",
				'callback' => [ $this->settings_controller, 'settings_default' ],
				'type'     => 'admin_post'
			],
			[
				'name'     => "{$this->prefix}_logs_settings_save",
				'callback' => [ $this->logs_controller, 'settings' ],
				'type'     => 'wp_ajax'
			],
		];

		$this->filters = [
			[
				'name'     => "{$this->prefix}_get_location_setting",
				'callback' => [ $this->locations_controller, 'get_settings' ],
				'args'     => 2
			],
			[
				'name'     => "{$this->prefix}_get_organization_data",
				'callback' => [ $this->locations_controller, 'get_organization_data' ],
				'args'     => 2
			],
		];
	}

	public function handle(): void {
		foreach ( $this->routes as $route ) {
			add_action( "{$route['type']}_{$route['name']}", $route['callback'] );
		}

		foreach ( $this->filters as $filter ) {
			add_filter( $filter['name'], $filter['callback'], 10, $filter['args'] );
		}

		add_action( 'sng_location_deleted', [$this->locations_controller, 'delete_settings'] );
	}
}