<?php

namespace SweepAndGo\App\Http\Controllers;

use SweepAndGo\App\View;
use SweepAndGo\Includes\Log;

class AddonController {

	public function addons() {
		View::render( 'admin/addons' );
	}

	public function install_addon() {
		global $wp_filesystem;

		// Requirements
		require_once ABSPATH . 'wp-admin/includes/file.php';
		require_once ABSPATH . 'wp-admin/includes/plugin.php';
		require_once ABSPATH . 'wp-admin/includes/class-wp-upgrader.php';
		require_once ABSPATH . 'wp-admin/includes/plugin-install.php';

		// Install addon
		$api                = new \stdClass();
		$api->name          = $_REQUEST['name'];
		$api->slug          = $_REQUEST['slug'];
		$api->version       = $_REQUEST['version'];
		$api->download_link = $_REQUEST['downloadLink'];
		$api->tested        = $_REQUEST['tested'];
		$api->nonce = wp_create_nonce( "install-plugin_{$_REQUEST['slug']}" );

		$skin     = new \WP_Ajax_Upgrader_Skin( $api );
		$upgrader = new \Plugin_Upgrader( $skin );

		$result = $upgrader->install($api->download_link);
		if ( is_wp_error( $result ) ) {
			$result->add_data( array( 'status' => 500 ) );
			// TODO Log
			wp_send_json_error( [ 'data' => $result ], 500 );
		}

		// This should be the same as $result above.
		if ( is_wp_error( $skin->result ) ) {
			$skin->result->add_data( array( 'status' => 500 ) );
			// TODO Log
			wp_send_json_error( $skin->result, 500 );
		}

		if ( $skin->get_errors()->has_errors() ) {
			// TODO Log
			$error = $skin->get_errors();
			$error->add_data( array( 'status' => 500 ) );

			wp_send_json_error( [ 'data' => $error ], 500 );
			wp_die();
		}

		if ( is_null( $result ) ) {
			// Pass through the error from WP_Filesystem if one was raised.
			if ( $wp_filesystem instanceof \WP_Filesystem_Base
			     && is_wp_error( $wp_filesystem->errors ) && $wp_filesystem->errors->has_errors()
			) {
				// TODO Log
				wp_send_json_error(new \WP_Error(
					'unable_to_connect_to_filesystem',
					$wp_filesystem->errors->get_error_message(),
					array( 'status' => 500 )
				), 500);
				wp_die();
			}
			// TODO Log
			wp_send_json_error(new \WP_Error(
				'unable_to_connect_to_filesystem',
				__( 'Unable to connect to the filesystem. Please confirm your credentials.' ),
				array( 'status' => 500 )
			), 500);
			wp_die();
		}

		$file = $upgrader->plugin_info();

		if ( ! $file ) {
			// TODO Log
			wp_send_json_error(new \WP_Error(
				'unable_to_determine_installed_plugin',
				__( 'Unable to determine what plugin was installed.' ),
				array( 'status' => 500 )
			), 500);
			wp_die();
		}

		$path          = WP_PLUGIN_DIR . '/' . $file;
		$data          = get_plugin_data( $path, false, false );
		$data['_file'] = $file;
		$data['status'] = 200;

		wp_send_json_success($data);
	}

	public function activate_addon() {
		// TODO nonce
		if ( ! is_plugin_active( $_REQUEST['plugin'] ) ){
			$activate = activate_plugin( $_REQUEST['plugin'] );
			if ( is_wp_error( $activate) ) {
				Log::error( "Error activating addon {$_REQUEST['plugin']}", $activate );
				wp_send_json_error($activate, 500);
			}
			wp_send_json_success();
		} else {
			wp_send_json_error('Error');
		}
	}

	public function deactivate_addon() {
		// TODO nonce
		if ( is_plugin_active( $_REQUEST['plugin'] ) ){
			deactivate_plugins( $_REQUEST['plugin'] );
			wp_send_json_success();
		} else {
			wp_send_json_error( [ 'message' => 'Plugin is not active!' ] );
		}
	}
}
