<?php

namespace SweepAndGo\App\Http\Controllers\Api;

use SweepAndGo\App\Helpers\LocationSetting;
use WP_REST_Request;
use WP_REST_Response;

class RestApiController extends \WP_REST_Controller {
	const API_VERSION = 'sng/v1';

	public array $webhooks = [];

	public function register_routes() {
		$routes = [
			'sng_webhooks' => [
				'method' => \WP_REST_Server::EDITABLE,
				'args' => [],
				'permission_callback' => [$this, 'validate_request'],
			]
		];
		foreach ($routes as $callback => $route) {
			register_rest_route( self::API_VERSION, $callback, [
				'methods' => $route['method'],
				'callback' => [$this, $callback],
				'permission_callback' => $route['permission_callback'],
				'args' => $route['args']
			] );
		}
	}

	public function sng_webhooks( WP_REST_Request $request ) {
		if ( isset( $this->webhooks[$request->get_param( 'type' )] ) ) {
			$webhook = $this->webhooks[$request->get_param( 'type' )];

			/**
			 * Multiple actions for a single webhook
			 */
			if ( is_array( $webhook['controller'] ) ) {
				for ( $i = 0; $i < count($webhook['controller'] ); $i++ ) {
					if ( method_exists( $webhook['controller'][$i], $webhook['action'][$i] ) ) {
						$controller = new $webhook['controller'][$i];
						call_user_func( [ $controller, $webhook['action'][$i] ], $request);
					}
				}
				return new WP_REST_Response(['message' => 'success'], 201);
			}

			/**
			 * Single action for a webhook
			 */
			if ( method_exists( $webhook['controller'], $webhook['action'] ) ) {
				$controller = new $webhook['controller'];
				return call_user_func( [ $controller, $webhook['action'] ], $request);
			} else {
				return false;
			}
		}

		return false;
	}

	public function register_webhook_actions( array $webhooks ) {
		$this->webhooks = array_merge_recursive( $webhooks, $this->webhooks );
	}

	public function default( WP_REST_Request $request ) {
		return new WP_REST_Response(['message'=>'success']);
	}

	/**
	 * Validate is the request really a webhook from Sweep&Go
	 * @param WP_REST_Request $request
	 * @return bool
	 */
	public function validate_request( WP_REST_Request $request ): bool {
		$algo = 'sha256';
		$content = $request->get_body();
		$signature = $request->get_header('signature');

		if ( ! $signature ) {
			return false;
		}

		$tokens = ( new LocationSetting() )->getTokens();

		$valid_request = false;
		foreach ( $tokens as $token ) {
			if ( $signature == hash_hmac($algo, $content, $token->token) ) {
				$valid_request = true;
			}
		}
		// Validate that the request came from Sweep&Go
		return $valid_request;
	}

	public function state_validation( WP_REST_Request $request ) {
		$wp_state = get_option('sng_oauth_state');

		if ( $wp_state != $request->get_param('state') ) {
			return false;
		}

		return true;
	}
}
