<?php

namespace SweepAndGo\App\Http\Controllers;

use SweepAndGo\App\View;

class SettingsController {
    public function settings() {
        do_action( 'sng_spinner_element' );
        $data = get_option( 'sng_settings', [] );
        View::render( 'admin/settings', $data );
    }

    public function settings_update() {
        if ( check_admin_referer( "update-sweeandgo-settings-" . get_current_user_id() ) &&
            current_user_can( 'manage_options' ) ) {
            $sng_settings = get_option( 'sng_settings', [] );
            $request = $_REQUEST;
            $message = '';
            $type = 'success';
            unset( $request[ '_wpnonce' ], $request[ '_wp_http_referer' ], $request[ 'action' ] );
            foreach ( $_FILES as $key => $image ) {
                if ( $image[ 'name' ] ) {
                    $upload = $this->upload_image( $image );
                    if ( $upload ) {
                        if ( is_array( $upload ) && isset( $upload[ 'icon' ] ) ) {
                            $request[ $key ] = $upload[ 'icon' ];
                        }
                    }
                }
            }

            if ( update_option( 'sng_settings', array_replace_recursive( $sng_settings, $request ) ) ) {
                $message .= "Updated successfully!<br/>";
            }

            set_transient( get_current_user_id() . 'sng_settings_action', [
                'type' => $type,
                'message' => __( $message, SNG_TEXT_DOMAIN )
            ], 10 );
            wp_safe_redirect( $_REQUEST[ '_wp_http_referer' ] );
            exit;
        }
    }

    public function settings_default(): bool {
        if ( check_admin_referer( "update-sweeandgo-settings-" . get_current_user_id() ) &&
            current_user_can( 'manage_options' ) ) {
            update_option( 'sng_settings', [] );
            set_transient( get_current_user_id() . 'sng_settings_action', [
                'type' => 'success',
                'message' => 'Settings reverted to default'
            ], 10 );
        }
        return wp_safe_redirect( $_REQUEST[ '_wp_http_referer' ] );
    }

    /**
     * Upload images to sweepandgo folder
     *
     * @param $image
     *
     * @return false|string|array - Image url, error or skip if there's no file in request
     */
    private function upload_image( $image ): false|string|array {
        $allowed_types = [ 'image/jpg', 'image/jpeg', 'image/webp', 'image/png', 'image/svg+xml' ];
        $upload_dir = wp_upload_dir();
        $sng_dir = str_replace( $upload_dir[ 'subdir' ], '', $upload_dir[ 'path' ] );

        // Create sweepandgo uploads folder if it doesn't exist
        if ( !file_exists( $sng_dir . SNG_UPLOAD_DIR ) ) {
            mkdir( $sng_dir . SNG_UPLOAD_DIR, 0755, true );
        }

        if ( isset( $image[ 'name' ] ) && $image[ 'name' ] != "" ) {
            $image[ 'name' ] = sanitize_file_name( basename( $image[ 'name' ] ) );
            $upload_to = $sng_dir . SNG_UPLOAD_DIR . "/{$image['name']}";

            // Check image type
            if ( !in_array( $image[ 'type' ], $allowed_types ) ) {
                return "File type {$image['type']} of {$image['name']} is not allowed";
            }

            // If file is succesfully uploaded
            if ( move_uploaded_file( $image[ 'tmp_name' ], $upload_to ) ) {
                $image[ 'icon' ] = "{$upload_dir['baseurl']}/sweepandgo/{$image['name']}";

                return $image;
            }
            else {
                return "Error uploading {$image['name']}";
            }
        }

        return false;
    }

    public function ab_testing() {
        if ( current_user_can( 'manage_options' ) ) {
            $post_a_id = $_REQUEST[ 'post' ];
            $post_a = (array)get_post( $post_a_id );
            unset( $post_a[ 'ID' ] );
            $b_post = wp_insert_post( $post_a );
            $post_meta = get_post_custom( $post_a_id );
            foreach ( $post_meta as $key => $value ) {
                add_post_meta( $b_post, $key, $value[ 0 ] );
            }

            add_post_meta( $b_post, '_sng_ab_testing', 'Version B' );
            add_post_meta( $post_a_id, '_sng_ab_testing', 'Version A' );
            if ( headers_sent() ) {
                printf( 'Created A/B pages. Please go back to <a href="%s">Sweep&Go Pages</a>', sanitize_url( admin_url( 'edit.php?post_type=sng_page' ) ) );
            }
            else {
                wp_safe_redirect( admin_url( 'edit.php?post_type=sng_page' ) );
            }
        }
    }

    public function ab_testing_finalize() {
        if ( current_user_can( 'manage_options' ) ) {
            $post = get_post( $_REQUEST[ 'post' ] );
            $post_version = get_post_meta( $post->ID, '_sng_ab_testing', true );
            if ( $post_version == 'Version A' ) {
                $post_b_version = 'Version B';
            }
            else {
                $post_b_version = 'Version A';
            }
            $post_b_args = [
                'post_type' => 'sng_page',
                'meta_query' => [
                    [
                        'key' => '_wp_page_template',
                        'value' => 'sng-client-onboarding-template.php',
                    ],
                    [
                        'key' => '_sng_ab_testing',
                        'value' => $post_b_version
                    ]
                ],
            ];
            $post_b = new \WP_Query( $post_b_args );
            wp_delete_post( $post_b->get_posts()[ 0 ]->ID );
            delete_post_meta( $post->ID, '_sng_ab_testing' );
            if ( headers_sent() ) {
                printf( 'AB Testing Finalized.  Please go back to <a href="%s">Sweep&Go Pages</a>', sanitize_url( admin_url( 'edit.php?post_type=sng_page' ) ) );
            }
            else {
                wp_safe_redirect( admin_url( 'edit.php?post_type=sng_page' ) );
            }
        }
    }
}