<?php

include_once( SNG_PLUGIN_DIR_PATH . '/views/admin/partials/admin_head.php' );
$addons_request = wp_remote_get( 'https://download.sweepandgo.com/live/addons.json' );
$response_code = wp_remote_retrieve_response_code( $addons_request );
if ( $response_code >= 200 && $response_code < 300 ) {
    $addons = json_decode( wp_remote_retrieve_body( $addons_request ) );
} else {
	throw new \Exception( 'Error fetching addons. Please contact the plugin developer.', 500);
}
?>

<div class="wrap sng-wrap">
    <div class="sng-container" style="padding: 1rem 2rem;">
        <div class="sng-addons-list">
            <?php foreach ( $addons as $addon ): ?>

            <div class="sng-card">
                <div class="sng-card-title">
                    <span class="sng-addon-name">
                        <?php echo sanitize_text_field( $addon->name ); ?>
                    </span>
                    <span class="sng-addon-version">
                        Version: <?php echo sanitize_text_field( $addon->version ); ?>
                    </span>
                </div>
                <div class="sng-card-body">
                    <div class="sng-addon-screenshot">
                        <img src="<?php echo $addon->screenshot; ?>"
                             alt="<?php echo sanitize_text_field( $addon->name ); ?> addon screenshot"/>
                    </div>
                    <p class="sng-addon-description">
	                    <?php echo sanitize_text_field( $addon->description ); ?>
                    </p>
                </div>
                <div class="sng-card-footer">
                    <div class="sng-addon-footer-info">
                        <?php
                        $plugin_version = @get_file_data(WP_PLUGIN_DIR . "/$addon->slug/$addon->slug.php", ['Version' => 'Version'], false);
                        if ( isset( $plugin_version ) && $plugin_version['Version'] ) {
	                        echo sprintf( '<span>Installed: <span class="sng-addon-version-installed %s">v%s%s</span></span>',
		                        ( $plugin_version['Version'] != $addon->version) ? 'sng-addon-update-available':'',
		                        $plugin_version['Version'],
		                        ( $plugin_version['Version'] != $addon->version) ? '(Update available)':'');
                        } else {
                            echo '<span>Not installed</span>';
                        }

                        ?>
                        <span class="sng-addon-price">Price: <?php echo ( $addon->price == 0 ) ? 'FREE': "\$$addon->price"; ?></span>
                    </div>
                    <div class="sng-addon-footer-actions">
                        <a class="sng-button-secondary" href="https://www.sweepandgo.com/" target="_blank">
                            Open in store
                        </a>
                        <?php
                            if ( count( get_plugin_files("$addon->slug/$addon->slug.php" ) ) == 1 ):
                        ?>
                            <button data-name="<?php echo sanitize_text_field( $addon->name ); ?>"
                                    data-slug="<?php echo sanitize_text_field( $addon->slug ); ?>"
                                    data-version="<?php echo sanitize_text_field( $addon->version ); ?>"
                                    data-tested="<?php echo sanitize_text_field( $addon->tested ); ?>"
                                    data-download-link="<?php echo sanitize_text_field( $addon->download_link ); ?>"
                                    class="sng-button-primary sng-install-addon" type="button">Install</button>
                        <?php
                        elseif ( ! is_plugin_active( "$addon->slug/$addon->slug.php" ) ):
                        ?>
                            <button class="sng-button-primary sng-activate-addon"
                                    type="button"
                                    data-plugin="<?php echo "$addon->slug/$addon->slug.php"; ?>">Activate</button>
                        <?php else: ?>
                            <button
                                    class="sng-button-primary sng-deactivate-addon"
                                    type="button"
                                    data-plugin="<?php echo "$addon->slug/$addon->slug.php"; ?>">Deactivate</button>
                        <?php endif; ?>
                    </div>
                </div>
            </div>
            <?php endforeach; ?>
        </div>
    </div>
</div>

<script>
    jQuery('.sng-install-addon').click( (evt) => {
        jQuery.post(ajaxurl, {
            action: 'sng_install_addon',
            ...evt.target.dataset
        }).then( response => {
            if ( response.success && response.data.status === 200 ) {
                evt.target.classList.add('success')
                evt.target.classList.remove('error')
                evt.target.innerText = "Installed"
                location.reload()
            } else {
                evt.target.classList.remove('success')
                evt.target.classList.add('error')
                evt.target.innerText = "Error"

                setTimeout(() => {
                    evt.target.classList.remove('error')
                    evt.target.innerText = "Install"
                }, 3500)
            }
            evt.target.classList.remove('loading')
        })
    })

    jQuery('.sng-activate-addon').click( evt => {
        jQuery.post(ajaxurl, {
            action: 'sng_activate_addon',
            ...evt.target.dataset
        }).then( response => {
            console.log(response)
            if ( response.success ) {
                location.reload()
            } else {
                evt.target.classList.add('error')
                evt.target.innerText = "Error"
                setTimeout(() => {
                    evt.target.classList.remove('error')
                    evt.target.innerText = "Activate"
                }, 3500)
            }
            evt.target.classList.remove('loading')
        }).fail(function(xhr, status, error) {
            evt.target.classList.remove('loading')
            console.log('response', xhr.responseJSON) // TODO Show error
            // error handling
        });
    })

    jQuery('.sng-deactivate-addon').click( evt => {
        jQuery.post(ajaxurl, {
            action: 'sng_deactivate_addon',
            ...evt.target.dataset
        }).then( response => {
            console.log(response)
            if ( response.success ) {
                location.reload()
            } else {
                evt.target.classList.add('error')
                evt.target.innerText = "Error"
                setTimeout(() => {
                    evt.target.classList.remove('error')
                    evt.target.innerText = "Deactivate"
                }, 3500)
            }
            evt.target.classList.remove('loading')
        }).fail(function(xhr, status, error) {
            evt.target.classList.remove('loading')
            console.log('xhr', xhr)
            console.log('status', status)
            console.log('error', error)
            // error handling
        });
    })
</script>